@@ -29,6 +29,7 @@
 	"  --dir                      Work directory to use\n" \
 	"  --table-jobs               Number of concurrent COPY jobs to run\n" \
 	"  --index-jobs               Number of concurrent CREATE INDEX jobs to run\n" \
+	"  --large-objects-jobs       Number of concurrent Large Objects jobs to run\n" \
 	"  --split-tables-larger-than Same-table concurrency size threshold\n" \
 	"  --drop-if-exists           On the target database, clean-up from a previous run first\n" \
 	"  --roles                    Also copy roles found on source to target\n" \
@@ -321,10 +322,13 @@ clone_and_follow(CopyDataSpec *copySpecs)
 	 * sequences, not the ones that were current when the main snapshot was
 	 * exported.
 	 */
-	if (!follow_reset_sequences(copySpecs, &streamSpecs))
+	if (success)
 	{
-		/* errors have already been logged */
-		exit(EXIT_CODE_TARGET);
+		if (!follow_reset_sequences(copySpecs, &streamSpecs))
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_TARGET);
+		}
 	}
 
 	/* make sure all sub-processes are now finished */
@@ -447,6 +451,8 @@ start_clone_process(CopyDataSpec *copySpecs, pid_t *pid)
 		case 0:
 		{
 			/* child process runs the command */
+			(void) set_ps_title("pgcopydb: clone");
+
 			log_notice("Starting the clone sub-process");
 
 			if (!cloneDB(copySpecs))
@@ -633,20 +639,20 @@ start_follow_process(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs,
 	 * been updated from a previous run of the command, and we might have
 	 * nothing to catch-up to when e.g. the endpos was reached already.
 	 */
-	CopyDBSentinel sentinel = { 0 };
+	CopyDBSentinel *sentinel = &(streamSpecs->sentinel);
 
-	if (!follow_init_sentinel(streamSpecs, &sentinel))
+	if (!follow_init_sentinel(streamSpecs, sentinel))
 	{
-		/* errors have already been logged */
+		log_error("Failed to initialise sentinel, see above for details");
 		return false;
 	}
 
-	if (sentinel.endpos != InvalidXLogRecPtr &&
-		sentinel.endpos <= sentinel.replay_lsn)
+	if (sentinel->endpos != InvalidXLogRecPtr &&
+		sentinel->endpos <= sentinel->replay_lsn)
 	{
 		log_info("Current endpos %X/%X was previously reached at %X/%X",
-				 LSN_FORMAT_ARGS(sentinel.endpos),
-				 LSN_FORMAT_ARGS(sentinel.replay_lsn));
+				 LSN_FORMAT_ARGS(sentinel->endpos),
+				 LSN_FORMAT_ARGS(sentinel->replay_lsn));
 
 		return true;
 	}
@@ -665,6 +671,7 @@ start_follow_process(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs,
 		case 0:
 		{
 			/* child process runs the command */
+			(void) set_ps_title("pgcopydb: follow");
 			log_notice("Starting the follow sub-process");
 
 			if (!follow_main_loop(copySpecs, streamSpecs))
@@ -697,6 +704,7 @@ cli_clone_follow_wait_subprocess(const char *name, pid_t pid)
 {
 	bool exited = false;
 	int returnCode = -1;
+	int sig = 0;
 
 	if (pid < 0)
 	{
@@ -706,7 +714,7 @@ cli_clone_follow_wait_subprocess(const char *name, pid_t pid)
 
 	while (!exited)
 	{
-		if (!follow_wait_pid(pid, &exited, &returnCode))
+		if (!follow_wait_pid(pid, &exited, &returnCode, &sig))
 		{
 			/* errors have already been logged */
 			return false;
