@@ -60,6 +60,27 @@ GUC serverSetttings[] = {
 };
 
 
+/*
+ * These parameters are added to the connection strings, unless the user has
+ * added them, allowing user-defined values to be taken into account.
+ */
+KeyVal connStringDefaults = {
+	.count = 4,
+	.keywords = {
+		"keepalives",
+		"keepalives_idle",
+		"keepalives_interval",
+		"keepalives_count"
+	},
+	.values = {
+		"1",
+		"10",
+		"10",
+		"60"
+	}
+};
+
+
 /*
  * copydb_init_tempdir initialises the file paths that are going to be used to
  * store temporary information while the pgcopydb process is running.
@@ -547,6 +568,10 @@ copydb_prepare_filepaths(CopyFilePaths *cfPaths,
 			"%s/wal_segment_size",
 			cfPaths->cdc.dir);
 
+	sformat(cfPaths->cdc.lsntrackingfile, MAXPGPATH,
+			"%s/lsn.json",
+			cfPaths->cdc.dir);
+
 	/*
 	 * Now prepare the "compare" files we need to compare schema and data
 	 * between the source and target instance.
@@ -589,14 +614,20 @@ copydb_prepare_dump_paths(CopyFilePaths *cfPaths, DumpPaths *dumpPaths)
 	sformat(dumpPaths->preFilename, MAXPGPATH, "%s/%s",
 			cfPaths->schemadir, "pre.dump");
 
+	sformat(dumpPaths->preListOutFilename, MAXPGPATH, "%s/%s",
+			cfPaths->schemadir, "pre-out.list");
+
+	sformat(dumpPaths->preListFilename, MAXPGPATH, "%s/%s",
+			cfPaths->schemadir, "pre-filtered.list");
+
 	sformat(dumpPaths->postFilename, MAXPGPATH, "%s/%s",
 			cfPaths->schemadir, "post.dump");
 
-	sformat(dumpPaths->preListFilename, MAXPGPATH, "%s/%s",
-			cfPaths->schemadir, "pre.list");
+	sformat(dumpPaths->postListOutFilename, MAXPGPATH, "%s/%s",
+			cfPaths->schemadir, "post-out.list");
 
 	sformat(dumpPaths->postListFilename, MAXPGPATH, "%s/%s",
-			cfPaths->schemadir, "post.list");
+			cfPaths->schemadir, "post-filtered.list");
 
 	return true;
 }
@@ -680,6 +711,7 @@ copydb_init_specs(CopyDataSpec *specs,
 
 		.tableJobs = options->tableJobs,
 		.indexJobs = options->indexJobs,
+		.lObjectJobs = options->lObjectJobs,
 
 		/* at the moment we don't have --vacuumJobs separately */
 		.vacuumJobs = options->tableJobs,
@@ -888,6 +920,10 @@ copydb_init_tablepaths(CopyFilePaths *cfPaths,
 			cfPaths->tbldir,
 			oid);
 
+	sformat(tablePaths->chksumFile, MAXPGPATH, "%s/%u.sum.json",
+			cfPaths->tbldir,
+			oid);
+
 	return true;
 }
 
