@@ -17,11 +17,21 @@
 #include "summary.h"
 
 
+/*
+ * These GUC settings are set with the SET command, and are meant to be
+ * controled by pgcopydb without a way for the user to override them.
+ */
 #define COMMON_GUC_SETTINGS \
 	{ "client_encoding", "'UTF-8'" }, \
-	{ "tcp_keepalives_idle", "'60s'" }, \
 	{ "extra_float_digits", "3" }, \
-	{ "statement_timeout", "0" }
+	{ "statement_timeout", "0" }, \
+	{ "default_transaction_read_only", "off" }
+/*
+ * These parameters are added to the connection strings, unless the user has
+ * added them, allowing user-defined values to be taken into account.
+ */
+extern KeyVal connStringDefaults;
+
 
 /*
  * pgcopydb creates System V OS level objects such as message queues and
@@ -123,7 +133,7 @@ typedef struct CopyTableDataPartSpec
 	int64_t min;                /* WHERE partKey >= min */
 	int64_t max;                /*   AND partKey  < max */
 
-	char partKey[NAMEDATALEN];
+	char partKey[PG_NAMEDATALEN];
 } CopyTableDataPartSpec;
 
 
@@ -216,7 +226,7 @@ typedef struct SourceFilterItem
  */
 typedef struct ExtensionReqs
 {
-	char extname[NAMEDATALEN];
+	char extname[PG_NAMEDATALEN];
 	char version[BUFSIZE];
 
 	UT_hash_handle hh;          /* makes this structure hashable */
@@ -259,6 +269,7 @@ typedef struct CopyDataSpec
 	int tableJobs;
 	int indexJobs;
 	int vacuumJobs;
+	int lObjectJobs;
 
 	SplitTableLargerThan splitTablesLargerThan;
 
@@ -268,6 +279,7 @@ typedef struct CopyDataSpec
 	Queue copyQueue;
 	Queue indexQueue;
 	Queue vacuumQueue;
+	Queue loQueue;
 
 	DumpPaths dumpPaths;
 
@@ -444,6 +456,7 @@ bool copydb_dump_source_schema(CopyDataSpec *specs,
 							   const char *snapshot,
 							   PostgresDumpSection section);
 bool copydb_target_prepare_schema(CopyDataSpec *specs);
+bool copydb_copy_database_properties(CopyDataSpec *specs);
 bool copydb_target_drop_tables(CopyDataSpec *specs);
 bool copydb_target_finalize_schema(CopyDataSpec *specs);
 
@@ -480,6 +493,7 @@ bool copydb_process_table_data(CopyDataSpec *specs);
 
 bool copydb_start_copy_supervisor(CopyDataSpec *specs);
 bool copydb_copy_supervisor(CopyDataSpec *specs);
+bool copydb_copy_supervisor_send_stop(CopyDataSpec *specs);
 bool copydb_start_table_data_workers(CopyDataSpec *specs);
 bool copydb_table_data_worker(CopyDataSpec *specs);
 
@@ -511,7 +525,14 @@ bool copydb_prepare_copy_query(CopyTableDataSpec *tableSpecs,
 
 /* blobs.c */
 bool copydb_start_blob_process(CopyDataSpec *specs);
-bool copydb_copy_blobs(CopyDataSpec *specs);
+
+bool copydb_blob_supervisor(CopyDataSpec *specs);
+bool copydb_start_blob_workers(CopyDataSpec *specs);
+bool copydb_blob_worker(CopyDataSpec *specs);
+bool copydb_queue_largeobject_metadata(CopyDataSpec *specs, uint64_t *count);
+bool copydb_copy_blob_by_oid(CopyDataSpec *specs, uint32_t oid);
+bool copydb_add_blob(CopyDataSpec *specs, uint32_t oid);
+bool copydb_send_lo_stop(CopyDataSpec *specs);
 
 /* vacuum.c */
 bool vacuum_start_workers(CopyDataSpec *specs);
@@ -524,4 +545,23 @@ bool vacuum_send_stop(CopyDataSpec *specs);
 bool prepare_summary_table(Summary *summary, CopyDataSpec *specs);
 bool print_summary(Summary *summary, CopyDataSpec *specs);
 
+/* compare.c */
+bool compare_schemas(CopyDataSpec *copySpecs);
+bool compare_data(CopyDataSpec *copySpecs);
+
+bool compare_start_workers(CopyDataSpec *copySpecs, Queue *queue);
+bool compare_queue_tables(CopyDataSpec *copySpecs, Queue *queue);
+bool compare_data_worker(CopyDataSpec *copySpecs, Queue *queue);
+bool compare_data_by_table_oid(CopyDataSpec *copySpecs, uint32_t oid);
+
+bool compare_read_tables_sums(CopyDataSpec *copySpecs);
+bool compare_table(CopyDataSpec *copySpecs, SourceTable *source);
+
+bool compare_fetch_schemas(CopyDataSpec *copySpecs,
+						   CopyDataSpec *sourceSpecs,
+						   CopyDataSpec *targetSpecs);
+
+bool compare_write_checksum(SourceTable *table, const char *filename);
+bool compare_read_checksum(SourceTable *table, const char *filename);
+
 #endif  /* COPYDB_H */
