@@ -53,6 +53,7 @@ typedef struct CDCPaths
 	char walsegsizefile[MAXPGPATH];   /* /tmp/pgcopydb/cdc/wal_segment_size */
 	char tlifile[MAXPGPATH];          /* /tmp/pgcopydb/cdc/tli */
 	char tlihistfile[MAXPGPATH];      /* /tmp/pgcopydb/cdc/tli.history */
+	char lsntrackingfile[MAXPGPATH];  /* /tmp/pgcopydb/cdc/lsn.json */
 } CDCPaths;
 
 
@@ -93,10 +94,12 @@ typedef struct DumpPaths
 	char extnspFilename[MAXPGPATH];  /* pg_dump --schema-only -n ... */
 
 	char preFilename[MAXPGPATH];     /* pg_dump --section=pre-data */
-	char preListFilename[MAXPGPATH]; /* pg_restore --list */
+	char preListOutFilename[MAXPGPATH]; /* pg_restore --list */
+	char preListFilename[MAXPGPATH]; /* pg_restore --use-list */
 
 	char postFilename[MAXPGPATH];     /* pg_dump --section=post-data */
-	char postListFilename[MAXPGPATH]; /* pg_restore --list */
+	char postListOutFilename[MAXPGPATH]; /* pg_restore --list */
+	char postListFilename[MAXPGPATH];    /* pg_restore --use-list */
 } DumpPaths;
 
 
@@ -107,6 +110,8 @@ typedef struct TableFilePaths
 	char doneFile[MAXPGPATH];    /* table done file (summary) */
 	char idxListFile[MAXPGPATH]; /* index oids list file */
 
+	char chksumFile[MAXPGPATH]; /* table checksum file */
+
 	char truncateDoneFile[MAXPGPATH];    /* table truncate done file */
 } TableFilePaths;
 
