@@ -118,7 +118,17 @@ copydb_fetch_schema_and_prepare_specs(CopyDataSpec *specs)
 				 "on the source database discards pg_table_size() caching");
 	}
 
-	/* first, are we doing extensions? */
+	/*
+	 * Grab the source database properties to be able to install them again on
+	 * the target, using ALTER DATABASE SET or ALTER USER IN DATABASE SET.
+	 */
+	if (!schema_list_database_properties(src, &(specs->catalog.gucsArray)))
+	{
+		log_error("Failed to fetch database properties, see above for details");
+		return false;
+	}
+
+	/* now, are we doing extensions? */
 	if (specs->section == DATA_SECTION_ALL ||
 		specs->section == DATA_SECTION_EXTENSION)
 	{
@@ -229,7 +239,9 @@ copydb_fetch_schema_and_prepare_specs(CopyDataSpec *specs)
 	}
 
 	/*
-	 * Now also fetch the list of schemas from the target database.
+	 * Now fetch the list of schemas and roles found in the target database.
+	 * The information is needed to fetch related database properties
+	 * (settings) when set to a specific role within that database.
 	 */
 	if (!copydb_prepare_target_catalog(specs))
 	{
@@ -293,20 +305,30 @@ copydb_prepare_table_specs(CopyDataSpec *specs, PGSQL *pgsql)
 		if (specs->splitTablesLargerThan.bytes > 0 &&
 			specs->splitTablesLargerThan.bytes <= source->bytes)
 		{
-			if (IS_EMPTY_STRING_BUFFER(source->partKey))
+			if (IS_EMPTY_STRING_BUFFER(source->partKey) &&
+				streq(source->amname, "heap"))
 			{
-				log_info("Table \"%s\".\"%s\" is %s large, "
+				log_info("Table %s is %s large "
 						 "which is larger than --split-tables-larger-than %s, "
-						 "but does not have a unique column of type integer "
-						 "(int2/int4/int8).",
-						 source->nspname,
-						 source->relname,
+						 "and does not have a unique column of type integer: "
+						 "splitting by CTID",
+						 source->qname,
 						 source->bytesPretty,
 						 specs->splitTablesLargerThan.bytesPretty);
 
-				log_warn("Skipping same-table concurrency for table \"%s\".\"%s\"",
-						 source->nspname,
-						 source->relname);
+				strlcpy(source->partKey, "ctid", sizeof(source->partKey));
+			}
+			else if (!streq(source->amname, "heap"))
+			{
+				log_info("Table %s is %s large "
+						 "which is larger than --split-tables-larger-than %s, "
+						 "does not have a unique column of type integer, "
+						 "and uses table access method \"%s\": "
+						 "same table concurrency is not enabled",
+						 source->qname,
+						 source->bytesPretty,
+						 specs->splitTablesLargerThan.bytesPretty,
+						 source->amname);
 
 				++copySpecsCount;
 				continue;
@@ -322,10 +344,9 @@ copydb_prepare_table_specs(CopyDataSpec *specs, PGSQL *pgsql)
 
 			if (source->partsArray.count > 1)
 			{
-				log_info("Table \"%s\".\"%s\" is %s large, "
-						 "%d COPY processes will be used, partitining on \"%s\".",
-						 source->nspname,
-						 source->relname,
+				log_info("Table %s is %s large, "
+						 "%d COPY processes will be used, partitioning on %s.",
+						 source->qname,
 						 source->bytesPretty,
 						 source->partsArray.count,
 						 source->partKey);
@@ -471,11 +492,10 @@ copydb_prepare_index_specs(CopyDataSpec *specs, PGSQL *pgsql)
 
 			if (table == NULL)
 			{
-				log_error("Failed to find table %u (\"%s\".\"%s\") "
+				log_error("Failed to find table %u (%s) "
 						  " in sourceTableHashByOid",
 						  oid,
-						  indexArray->array[i].tableNamespace,
-						  indexArray->array[i].tableRelname);
+						  indexArray->array[i].tableQname);
 				return false;
 			}
 
@@ -1058,12 +1078,24 @@ copydb_prepare_target_catalog(CopyDataSpec *specs)
 		return true;
 	}
 
-	if (!pgsql_init(&dst, specs->connStrings.target_pguri, PGSQL_CONN_SOURCE))
+	if (!pgsql_init(&dst, specs->connStrings.target_pguri, PGSQL_CONN_TARGET))
 	{
 		/* errors have already been logged */
 		return false;
 	}
 
+	if (!pgsql_begin(&dst))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	/*
+	 * First, get a list of the schema that already exist on the target system.
+	 * Some extensions scripts create schema in a way that does not register a
+	 * dependency between the extension and the schema (using a DO $$ ... $$
+	 * block for instance), and there is no CREATE SCHEMA IF NOT EXISTS.
+	 */
 	SourceSchema *schemaHashByName = NULL;
 	SourceSchemaArray *schemaArray = &(specs->targetCatalog.schemaArray);
 
@@ -1083,6 +1115,39 @@ copydb_prepare_target_catalog(CopyDataSpec *specs)
 
 	specs->targetCatalog.schemaHashByName = schemaHashByName;
 
+	/*
+	 * Now fetch a list of roles that exist on the target system, so that we
+	 * may copy the database properties including specific to roles when they
+	 * exist on the target system:
+	 *
+	 *  ALTER DATABASE foo SET name = value;
+	 *  ALTER ROLE bob IN DATABASE foo SET name = value;
+	 */
+	SourceRole *rolesHashByName = NULL;
+	SourceRoleArray *rolesArray = &(specs->targetCatalog.rolesArray);
+
+	if (!schema_list_roles(&dst, rolesArray))
+	{
+		log_error("Failed to list roles on the target database");
+		return false;
+	}
+
+	for (int i = 0; i < rolesArray->count; i++)
+	{
+		SourceRole *role = &(rolesArray->array[i]);
+		size_t len = strlen(role->rolname);
+
+		HASH_ADD(hh, rolesHashByName, rolname, len, role);
+	}
+
+	specs->targetCatalog.rolesHashByName = rolesHashByName;
+
+	if (!pgsql_commit(&dst))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
 	return true;
 }
 
@@ -1104,28 +1169,33 @@ copydb_schema_already_exists(CopyDataSpec *specs,
 		return false;
 	}
 
-	char *name = strdup(restoreListName + 2);
+	const char *start = restoreListName + 2;
+	char *end = strchr(start, ' ');
 
-	if (name == NULL)
-	{
-		log_error(ALLOCATION_FAILED_ERROR);
-		return false;
-	}
-
-	char *spc = strchr(name, ' ');
-
-	if (spc == NULL)
+	if (end == NULL)
 	{
 		log_error("Failed to parse restore list name \"%s\"", restoreListName);
 		return false;
 	}
 
-	*spc = '\0';
+	/* skip the end space itself pointed */
+	int len = end - start;
+	char nspname[NAMEDATALEN] = { 0 };
+	size_t bytes = sformat(nspname, sizeof(nspname), "%.*s", len, start);
+
+	if (bytes >= sizeof(nspname))
+	{
+		log_error("Failed to parse schema name \"%s\" (%d bytes long), "
+				  "pgcopydb and Postgres only support names up to %lu bytes",
+				  nspname,
+				  len,
+				  sizeof(nspname));
+		return false;
+	}
 
 	SourceSchema *schema = NULL;
-	size_t len = strlen(name);
 
-	HASH_FIND(hh, schemaHashByName, name, len, schema);
+	HASH_FIND(hh, schemaHashByName, nspname, len, schema);
 
 	*exists = schema != NULL;
 
