@@ -419,6 +419,7 @@ read_from_stream(FILE *stream, ReadFromStreamContext *context)
 {
 	int countFdsReadyToRead, nfds; /* see man select(2) */
 	fd_set readFileDescriptorSet;
+	fd_set exceptFileDescriptorSet;
 
 	context->fd = fileno(stream);
 	nfds = context->fd + 1;
@@ -442,15 +443,29 @@ read_from_stream(FILE *stream, ReadFromStreamContext *context)
 		FD_ZERO(&readFileDescriptorSet);
 		FD_SET(context->fd, &readFileDescriptorSet);
 
+		FD_ZERO(&exceptFileDescriptorSet);
+		FD_SET(context->fd, &exceptFileDescriptorSet);
+
 		countFdsReadyToRead =
-			select(nfds, &readFileDescriptorSet, NULL, NULL, &timeout);
+			select(nfds,
+				   &readFileDescriptorSet, NULL, &exceptFileDescriptorSet,
+				   &timeout);
 
 		if (countFdsReadyToRead == -1)
 		{
+			log_debug("countFdsReadyToRead == -1");
+
 			if (errno == EINTR || errno == EAGAIN)
 			{
-				if (asked_to_stop || asked_to_stop_fast || asked_to_quit)
+				log_debug("received EINTR or EAGAIN");
+
+				if (asked_to_quit)
 				{
+					/*
+					 * When asked_to_stop || asked_to_stop_fast still continue
+					 * reading through EOF on the input stream, then quit
+					 * normally.
+					 */
 					doneReading = true;
 				}
 
@@ -464,6 +479,14 @@ read_from_stream(FILE *stream, ReadFromStreamContext *context)
 			}
 		}
 
+		if (FD_ISSET(context->fd, &exceptFileDescriptorSet))
+		{
+			log_error("Failed to select on file descriptor %d: "
+					  "an exceptional condition happened",
+					  context->fd);
+			return false;
+		}
+
 		/*
 		 * When asked_to_stop || asked_to_stop_fast still continue reading
 		 * through EOF on the input stream, then quit normally. Here when
@@ -472,10 +495,10 @@ read_from_stream(FILE *stream, ReadFromStreamContext *context)
 		 */
 		if (countFdsReadyToRead == 0)
 		{
-			if (asked_to_stop || asked_to_stop_fast || asked_to_quit)
+			if (asked_to_quit)
 			{
 				doneReading = true;
-				log_info("read_from_stream was asked to stop or quit");
+				log_notice("read_from_stream was asked to quit");
 			}
 
 			continue;
@@ -523,8 +546,9 @@ read_from_stream(FILE *stream, ReadFromStreamContext *context)
 			/* if the buffer doesn't terminate with \n it's a partial read */
 			bool partialRead = buf[bytes - 1] != '\n';
 
-			char *lines[BUFSIZE] = { 0 };
-			int lineCount = splitLines(buf, lines, BUFSIZE);
+			int count = countLines(buf);
+			char **lines = (char **) calloc(count, sizeof(char *));
+			int lineCount = splitLines(buf, lines, count);
 
 			log_trace("read_from_stream read %6zu bytes in %d lines %s[%lld]",
 					  bytes,
@@ -662,6 +686,7 @@ read_from_stream(FILE *stream, ReadFromStreamContext *context)
 				}
 			}
 
+			free(lines);
 			free(buf);
 		}
 
@@ -1276,11 +1301,21 @@ set_ps_title(const char *title)
 		return;
 	}
 
-	int n = sformat(ps_buffer, ps_buffer_size, "%s", title);
-
 	/* pad our process title string */
-	for (size_t i = n; i < ps_buffer_size; i++)
+	int n = strlen(title);
+
+	for (size_t i = 0; i < ps_buffer_size; i++)
 	{
-		*(ps_buffer + i) = '\0';
+		if (i < n)
+		{
+			*(ps_buffer + i) = title[i];
+		}
+		else
+		{
+			*(ps_buffer + i) = '\0';
+		}
 	}
+
+	/* make sure we have an \0 at the end of the ps_buffer */
+	*(ps_buffer + ps_buffer_size - 1) = '\0';
 }
