@@ -14,6 +14,7 @@
 #include "cli_root.h"
 #include "ld_stream.h"
 #include "log.h"
+#include "progress.h"
 #include "signals.h"
 
 
@@ -153,7 +154,7 @@ follow_init_sentinel(StreamSpecs *specs, CopyDBSentinel *sentinel)
 	if (!pgsql_begin(&pgsql))
 	{
 		/* errors have already been logged */
-		exit(EXIT_CODE_SOURCE);
+		return false;
 	}
 
 	if (specs->endpos != InvalidXLogRecPtr)
@@ -186,7 +187,7 @@ follow_init_sentinel(StreamSpecs *specs, CopyDBSentinel *sentinel)
  * values from the pgcopydb.sentinel table on the source database.
  */
 bool
-follow_get_sentinel(StreamSpecs *specs, CopyDBSentinel *sentinel)
+follow_get_sentinel(StreamSpecs *specs, CopyDBSentinel *sentinel, bool verbose)
 {
 	PGSQL pgsql = { 0 };
 
@@ -216,16 +217,19 @@ follow_get_sentinel(StreamSpecs *specs, CopyDBSentinel *sentinel)
 				 LSN_FORMAT_ARGS(sentinel->replay_lsn),
 				 LSN_FORMAT_ARGS(sentinel->endpos));
 	}
-	else if (sentinel->endpos != InvalidXLogRecPtr)
+	else if (verbose)
 	{
-		log_info("Current sentinel replay_lsn is %X/%X, endpos is %X/%X",
-				 LSN_FORMAT_ARGS(sentinel->replay_lsn),
-				 LSN_FORMAT_ARGS(sentinel->endpos));
-	}
-	else
-	{
-		log_info("Current sentinel replay_lsn is %X/%X",
-				 LSN_FORMAT_ARGS(sentinel->replay_lsn));
+		if (sentinel->endpos != InvalidXLogRecPtr)
+		{
+			log_info("Current sentinel replay_lsn is %X/%X, endpos is %X/%X",
+					 LSN_FORMAT_ARGS(sentinel->replay_lsn),
+					 LSN_FORMAT_ARGS(sentinel->endpos));
+		}
+		else if (sentinel->replay_lsn != InvalidXLogRecPtr)
+		{
+			log_info("Current sentinel replay_lsn is %X/%X",
+					 LSN_FORMAT_ARGS(sentinel->replay_lsn));
+		}
 	}
 
 	return true;
@@ -265,6 +269,16 @@ follow_main_loop(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs)
 		return false;
 	}
 
+	/*
+	 * Read the catalogs from the source table from on-file disk if
+	 * the schema dump has been done already.
+	 */
+	if (copySpecs->dirState.schemaDumpIsDone && !copydb_parse_schema_json_file(copySpecs))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
 	/*
 	 * In case of successful exit from the follow sub-processes, we
 	 * switch back and forth between CATCHUP and REPLAY modes and
@@ -285,7 +299,8 @@ follow_main_loop(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs)
 	{
 		if (!followDB(copySpecs, streamSpecs))
 		{
-			/* errors have already been logged */
+			log_error("Failed to follow changes from source, "
+					  "see above for details");
 			return false;
 		}
 
@@ -305,6 +320,11 @@ follow_main_loop(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs)
 
 		if (done)
 		{
+			log_info("Follow mode is now done, "
+					 "reached replay_lsn %X/%X with endpos %X/%X",
+					 LSN_FORMAT_ARGS(streamSpecs->sentinel.endpos),
+					 LSN_FORMAT_ARGS(streamSpecs->sentinel.replay_lsn));
+
 			return true;
 		}
 
@@ -312,13 +332,6 @@ follow_main_loop(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs)
 		previousMode = currentMode;
 		currentMode = modeArray[++loop % count];
 
-		/* and re-init our streamSpecs for the new mode */
-		if (!stream_init_for_mode(streamSpecs, currentMode))
-		{
-			/* errors have already been logged */
-			return false;
-		}
-
 		/*
 		 * Whatever the current/previous mode was, we need to
 		 * ensure to catch-up with files on-disk before switching
@@ -345,13 +358,27 @@ follow_main_loop(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs)
 
 		if (done)
 		{
+			log_info("Follow mode is now done, "
+					 "reached replay_lsn %X/%X with endpos %X/%X",
+					 LSN_FORMAT_ARGS(streamSpecs->sentinel.endpos),
+					 LSN_FORMAT_ARGS(streamSpecs->sentinel.replay_lsn));
+
 			return true;
 		}
 
 		log_info("Restarting logical decoding follower in %s mode",
 				 LogicalStreamModeToString(currentMode));
+
+		/* and re-init our streamSpecs for the new mode */
+		if (!stream_init_for_mode(streamSpecs, currentMode))
+		{
+			/* errors have already been logged */
+			return false;
+		}
 	}
 
+	/* keep compiler happy */
+	log_warn("BUG: follow_main_loop reached out of loop");
 	return true;
 }
 
@@ -362,23 +389,24 @@ follow_main_loop(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs)
 bool
 follow_reached_endpos(StreamSpecs *streamSpecs, bool *done)
 {
-	CopyDBSentinel sentinel = { 0 };
+	bool verbose = true;
+	CopyDBSentinel *sentinel = &(streamSpecs->sentinel);
 
-	if (!follow_get_sentinel(streamSpecs, &sentinel))
+	if (!follow_get_sentinel(streamSpecs, sentinel, verbose))
 	{
-		/* errors have already been logged */
+		log_error("Failed to get sentinel values");
 		return false;
 	}
 
-	if (sentinel.endpos != InvalidXLogRecPtr &&
-		sentinel.endpos <= sentinel.replay_lsn)
+	if (sentinel->endpos != InvalidXLogRecPtr &&
+		sentinel->endpos <= sentinel->replay_lsn)
 	{
 		/* follow_get_sentinel logs replay_lsn and endpos already */
 		*done = true;
 
 		log_info("Current endpos %X/%X has been reached at %X/%X",
-				 LSN_FORMAT_ARGS(sentinel.endpos),
-				 LSN_FORMAT_ARGS(sentinel.replay_lsn));
+				 LSN_FORMAT_ARGS(sentinel->endpos),
+				 LSN_FORMAT_ARGS(sentinel->replay_lsn));
 	}
 
 	return true;
@@ -430,21 +458,51 @@ follow_prepare_mode_switch(StreamSpecs *streamSpecs,
 			log_notice("Processing %lld messages from the transform queue",
 					   (long long) qStats.msg_qnum);
 
-			if (!stream_transform_from_queue(streamSpecs))
+			FollowSubProcess *transform = &(streamSpecs->transform);
+
+			if (!follow_start_subprocess(streamSpecs, transform))
 			{
-				/* errors have already been logged */
+				log_error("Failed to start the transform process");
+				return false;
+			}
+
+			if (!follow_wait_subprocesses(streamSpecs))
+			{
+				log_error("Failed to transform %lld messages from the queue, "
+						  "see above for details",
+						  (long long) qStats.msg_qnum);
 				return false;
 			}
 		}
 	}
 
-	/* then catch-up with what's been stream and transformed already */
-	if (!stream_apply_catchup(streamSpecs))
+	/*
+	 * Then catch-up with what's been stream and transformed already, which
+	 * means replaying the files that have already been prepared on-disk.
+	 */
+	LogicalStreamMode mode = streamSpecs->mode;
+	streamSpecs->mode = STREAM_MODE_CATCHUP;
+
+	FollowSubProcess *catchup = &(streamSpecs->catchup);
+
+	if (!follow_start_subprocess(streamSpecs, catchup))
 	{
-		/* errors have already been logged */
+		streamSpecs->mode = mode;
+		log_error("Failed to start the %s process", catchup->name);
 		return false;
 	}
 
+	if (!follow_wait_subprocesses(streamSpecs))
+	{
+		streamSpecs->mode = mode;
+		log_error("Failed to catchup with on-disk files, "
+				  "see above for details");
+		return false;
+	}
+
+	/* re-install the streamSpecs->mode as it was before getting there */
+	streamSpecs->mode = mode;
+
 	return true;
 }
 
@@ -468,6 +526,19 @@ followDB(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs)
 		return false;
 	}
 
+	/*
+	 * Before starting sub-processes, clean-up intermediate files from previous
+	 * round. Here that's the stream context with WAL segment size and timeline
+	 * history, which are fetched from the source server to compute WAL file
+	 * names. The current timeline can only change at a server restart or a
+	 * failover, both with trigger a reconnect.
+	 */
+	if (!stream_cleanup_context(streamSpecs))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
 	/*
 	 * Prepare the sub-process communication mechanisms, when needed:
 	 *
@@ -568,25 +639,44 @@ follow_start_prefetch(StreamSpecs *specs)
 	if (specs->mode == STREAM_MODE_REPLAY)
 	{
 		/* arrange to write to the receive-transform pipe */
+		specs->stdIn = false;
+		specs->stdOut = true;
+
 		specs->out = fdopen(specs->pipe_rt[1], "a");
 
 		/* close pipe ends we're not using */
 		close_fd_or_exit(specs->pipe_rt[0]);
 		close_fd_or_exit(specs->pipe_ta[0]);
 		close_fd_or_exit(specs->pipe_ta[1]);
-	}
 
-	if (!stream_cleanup_context(specs))
+		/* switch out stream from block buffered to line buffered mode */
+		if (setvbuf(specs->out, NULL, _IOLBF, 0) != 0)
+		{
+			log_error("Failed to set out stream to line buffered mode: %m");
+			return false;
+		}
+
+		bool success = startLogicalStreaming(specs);
+
+		close_fd_or_exit(specs->pipe_rt[1]);
+
+		log_info("Prefetch process has terminated");
+
+		return success;
+	}
+	else
 	{
-		/* errors have already been logged */
-		return false;
+		specs->stdIn = false;
+		specs->stdOut = false;
+
+		bool success = startLogicalStreaming(specs);
+
+		log_info("Prefetch process has terminated");
+
+		return success;
 	}
 
-	bool success = startLogicalStreaming(specs);
-
-	close_fd_or_exit(specs->pipe_rt[1]);
-
-	return success;
+	return true;
 }
 
 
@@ -609,6 +699,9 @@ follow_start_transform(StreamSpecs *specs)
 		 * Arrange to read from receive-transform pipe and write to the
 		 * transform-apply pipe.
 		 */
+		specs->stdIn = true;
+		specs->stdOut = true;
+
 		specs->in = fdopen(specs->pipe_rt[0], "r");
 		specs->out = fdopen(specs->pipe_ta[1], "a");
 
@@ -616,6 +709,13 @@ follow_start_transform(StreamSpecs *specs)
 		close_fd_or_exit(specs->pipe_rt[1]);
 		close_fd_or_exit(specs->pipe_ta[0]);
 
+		/* switch out stream from block buffered to line buffered mode */
+		if (setvbuf(specs->out, NULL, _IOLBF, 0) != 0)
+		{
+			log_error("Failed to set out stream to line buffered mode: %m");
+			return false;
+		}
+
 		bool success = stream_transform_stream(specs);
 
 		close_fd_or_exit(specs->pipe_rt[0]);
@@ -631,6 +731,9 @@ follow_start_transform(StreamSpecs *specs)
 		 * internal message queue and batch processes one file at a
 		 * time.
 		 */
+		specs->stdIn = false;
+		specs->stdOut = false;
+
 		return stream_transform_worker(specs);
 	}
 
@@ -651,6 +754,9 @@ follow_start_catchup(StreamSpecs *specs)
 	if (specs->mode == STREAM_MODE_REPLAY)
 	{
 		/* arrange to read from the transform-apply pipe */
+		specs->stdIn = true;
+		specs->stdOut = false;
+
 		specs->in = fdopen(specs->pipe_ta[0], "r");
 
 		/* close pipe ends we're not using */
@@ -672,6 +778,9 @@ follow_start_catchup(StreamSpecs *specs)
 		 * current LSN on the target database origin tracking system to
 		 * open the right SQL file and apply statements from there.
 		 */
+		specs->stdIn = false;
+		specs->stdOut = false;
+
 		return stream_apply_catchup(specs);
 	}
 
@@ -710,6 +819,13 @@ follow_start_subprocess(StreamSpecs *specs, FollowSubProcess *subprocess)
 		case 0:
 		{
 			/* child process runs the command */
+			char psTitle[BUFSIZE] = { 0 };
+
+			sformat(psTitle, sizeof(psTitle), "pgcopydb: follow %s",
+					subprocess->name);
+
+			(void) set_ps_title(psTitle);
+
 			log_notice("Starting the %s sub-process", subprocess->name);
 
 			if (!(subprocess->command)(specs))
@@ -800,7 +916,8 @@ follow_wait_subprocesses(StreamSpecs *specs)
 			/* follow_wait_pid is non-blocking: uses WNOHANG */
 			if (!follow_wait_pid(processArray[i]->pid,
 								 &(processArray[i]->exited),
-								 &(processArray[i]->returnCode)))
+								 &(processArray[i]->returnCode),
+								 &(processArray[i]->sig)))
 			{
 				/* errors have already been logged */
 				return false;
@@ -815,13 +932,33 @@ follow_wait_subprocesses(StreamSpecs *specs)
 
 				if (processArray[i]->returnCode == 0)
 				{
-					sformat(details, sizeof(details), "successfully");
+					if (processArray[i]->sig == 0)
+					{
+						sformat(details, sizeof(details), "successfully");
+					}
+					else
+					{
+						sformat(details, sizeof(details),
+								"successfully after signal %s",
+								signal_to_string(processArray[i]->sig));
+					}
 				}
 				else
 				{
 					logLevel = LOG_ERROR;
-					sformat(details, sizeof(details), "with error code %d",
-							processArray[i]->returnCode);
+
+					if (processArray[i]->sig == 0)
+					{
+						sformat(details, sizeof(details), "with error code %d",
+								processArray[i]->returnCode);
+					}
+					else
+					{
+						sformat(details, sizeof(details),
+								"with error code %d and signal %s",
+								processArray[i]->returnCode,
+								signal_to_string(processArray[i]->sig));
+					}
 				}
 
 				log_level(logLevel,
@@ -841,10 +978,38 @@ follow_wait_subprocesses(StreamSpecs *specs)
 				 * Otherwise there is no reason for the other processes to
 				 * stop, and we're missing one: terminate every one and handle
 				 * at the caller.
+				 *
+				 * We need to first update current sentinel values (endpos).
 				 */
+				if (!follow_get_sentinel(specs, &(specs->sentinel), false))
+				{
+					/* continue without updated endpos */
+					log_warn("Failed to get sentinel values");
+				}
+
 				if (processArray[i]->returnCode != 0 ||
 					specs->endpos == InvalidXLogRecPtr)
 				{
+					char endposStatus[BUFSIZE] = { 0 };
+
+					if (specs->endpos == InvalidXLogRecPtr)
+					{
+						sformat(endposStatus, sizeof(endposStatus), "unset");
+					}
+					else
+					{
+						sformat(endposStatus, sizeof(endposStatus),
+								"set to %X/%X",
+								LSN_FORMAT_ARGS(specs->endpos));
+					}
+
+					log_notice("Process %s has exited with return code %d, "
+							   "and endpos is %s: "
+							   "terminating other processes",
+							   processArray[i]->name,
+							   processArray[i]->returnCode,
+							   endposStatus);
+
 					if (!follow_terminate_subprocesses(specs))
 					{
 						log_error("Failed to terminate other subprocesses, "
@@ -913,7 +1078,7 @@ follow_terminate_subprocesses(StreamSpecs *specs)
  * follow_wait_pid waits for a given known sub-process.
  */
 bool
-follow_wait_pid(pid_t subprocess, bool *exited, int *returnCode)
+follow_wait_pid(pid_t subprocess, bool *exited, int *returnCode, int *sig)
 {
 	int status = 0;
 
@@ -932,7 +1097,9 @@ follow_wait_pid(pid_t subprocess, bool *exited, int *returnCode)
 			if (errno == ECHILD)
 			{
 				/* no more childrens */
+				*sig = 0;
 				*exited = true;
+				*returnCode = -1;
 				return true;
 			}
 			else
@@ -950,7 +1117,9 @@ follow_wait_pid(pid_t subprocess, bool *exited, int *returnCode)
 			 * We're using WNOHANG, 0 means there are no stopped or
 			 * exited children, it's all good.
 			 */
+			*sig = 0;
 			*exited = false;
+			*returnCode = -1;
 			break;
 		}
 
@@ -963,9 +1132,15 @@ follow_wait_pid(pid_t subprocess, bool *exited, int *returnCode)
 				return false;
 			}
 
+			*sig = 0;
 			*exited = true;
 			*returnCode = WEXITSTATUS(status);
 
+			if (WIFSIGNALED(status))
+			{
+				*sig = WTERMSIG(status);
+			}
+
 			break;
 		}
 	}
