@@ -45,13 +45,15 @@ copydb_start_index_workers(CopyDataSpec *specs)
 		{
 			case -1:
 			{
-				log_error("Failed to fork a worker process: %m");
+				log_error("Failed to fork a create index worker process: %m");
 				return false;
 			}
 
 			case 0:
 			{
 				/* child process runs the command */
+				(void) set_ps_title("pgcopydb: create index worker");
+
 				if (!copydb_index_worker(specs))
 				{
 					/* errors have already been logged */
@@ -90,22 +92,24 @@ copydb_index_worker(CopyDataSpec *specs)
 
 	while (!stop)
 	{
-		QMessage mesg = { 0 };
-		bool recv_ok = queue_receive(&(specs->indexQueue), &mesg);
+		QMessage *mesg = (QMessage *) calloc(1, sizeof(QMessage));
+		bool recv_ok = queue_receive(&(specs->indexQueue), mesg);
 
 		if (asked_to_stop || asked_to_stop_fast || asked_to_quit)
 		{
 			log_error("CREATE INDEX worker has been interrupted");
+			free(mesg);
 			return false;
 		}
 
 		if (!recv_ok)
 		{
 			/* errors have already been logged */
+			free(mesg);
 			return false;
 		}
 
-		switch (mesg.type)
+		switch (mesg->type)
 		{
 			case QMSG_TYPE_STOP:
 			{
@@ -116,17 +120,19 @@ copydb_index_worker(CopyDataSpec *specs)
 
 			case QMSG_TYPE_INDEXOID:
 			{
-				if (!copydb_create_index_by_oid(specs, mesg.data.oid))
+				if (!copydb_create_index_by_oid(specs, mesg->data.oid))
 				{
+					++errors;
+
+					log_error("Failed to create index with oid %u, "
+							  "see above for details",
+							  mesg->data.oid);
+
 					if (specs->failFast)
 					{
-						log_error("Failed to create index with oid %u, "
-								  "see above for details",
-								  mesg.data.oid);
+						free(mesg);
 						return false;
 					}
-
-					++errors;
 				}
 				break;
 			}
@@ -134,11 +140,12 @@ copydb_index_worker(CopyDataSpec *specs)
 			default:
 			{
 				log_error("Received unknown message type %ld on index queue %d",
-						  mesg.type,
+						  mesg->type,
 						  specs->indexQueue.qId);
 				break;
 			}
 		}
+		free(mesg);
 	}
 
 	bool success = (stop == true && errors == 0);
@@ -190,11 +197,10 @@ copydb_create_index_by_oid(CopyDataSpec *specs, uint32_t indexOid)
 
 	if (table == NULL)
 	{
-		log_error("Failed to find table %u (\"%s\".\"%s\") "
+		log_error("Failed to find table %u (%s) "
 				  " in sourceTableHashByOid",
 				  oid,
-				  index->tableNamespace,
-				  index->tableRelname);
+				  index->tableQname);
 		return false;
 	}
 
@@ -206,12 +212,10 @@ copydb_create_index_by_oid(CopyDataSpec *specs, uint32_t indexOid)
 		return false;
 	}
 
-	log_trace("copydb_create_index_by_oid: %u \"%s.%s\" on \"%s\".\"%s\"",
+	log_trace("copydb_create_index_by_oid: %u %s on %s",
 			  indexOid,
-			  index->indexNamespace,
-			  index->indexRelname,
-			  table->nspname,
-			  table->relname);
+			  index->indexQname,
+			  table->qname);
 
 	/*
 	 * Add IF NOT EXISTS clause when the --resume option has been used, or when
@@ -262,9 +266,8 @@ copydb_create_index_by_oid(CopyDataSpec *specs, uint32_t indexOid)
 
 		if (!copydb_create_constraints(specs, table))
 		{
-			log_error("Failed to create constraints for table \"%s\".\"%s\"",
-					  table->nspname,
-					  table->relname);
+			log_error("Failed to create constraints for table %s",
+					  table->qname);
 			return false;
 		}
 
@@ -272,9 +275,8 @@ copydb_create_index_by_oid(CopyDataSpec *specs, uint32_t indexOid)
 		{
 			if (!vacuum_add_table(specs, table->oid))
 			{
-				log_error("Failed to queue VACUUM ANALYZE \"%s\".\"%s\" [%u]",
-						  table->nspname,
-						  table->relname,
+				log_error("Failed to queue VACUUM ANALYZE %s [%u]",
+						  table->qname,
 						  table->oid);
 				return false;
 			}
@@ -344,9 +346,8 @@ copydb_table_indexes_are_done(CopyDataSpec *specs,
 		if (!create_table_index_file(table, tablePaths->idxListFile))
 		{
 			/* this only means summary is missing some indexing information */
-			log_warn("Failed to create table \"%s\".\"%s\" index list file \"%s\"",
-					 table->nspname,
-					 table->relname,
+			log_warn("Failed to create table %s index list file \"%s\"",
+					 table->qname,
 					 tablePaths->idxListFile);
 		}
 	}
@@ -379,9 +380,8 @@ copydb_add_table_indexes(CopyDataSpec *specs, CopyTableDataSpec *tableSpecs)
 			.data.oid = index->indexOid
 		};
 
-		log_trace("Queueing index \"%s\".\"%s\" [%u] for table %s [%u]",
-				  index->indexNamespace,
-				  index->indexRelname,
+		log_trace("Queueing index %s [%u] for table %s [%u]",
+				  index->indexQname,
 				  mesg.data.oid,
 				  tableSpecs->sourceTable->qname,
 				  tableSpecs->sourceTable->oid);
@@ -572,6 +572,8 @@ copydb_start_index_processes(CopyDataSpec *specs,
 			case 0:
 			{
 				/* child process runs the command */
+				(void) set_ps_title("pgcopydb: create index worker");
+
 				if (!copydb_start_index_process(specs,
 												indexArray,
 												indexPathsArray))
@@ -717,12 +719,11 @@ copydb_create_index(const char *pguri,
 	{
 		skipCreateIndex = true;
 		log_notice("Skipping concurrent build of index "
-				   "\"%s\" for constraint %s on \"%s\".\"%s\", "
+				   "%s for constraint %s on %s, "
 				   "it is not a UNIQUE or a PRIMARY constraint",
-				   index->indexRelname,
+				   index->indexQname,
 				   index->constraintDef,
-				   index->tableNamespace,
-				   index->tableRelname);
+				   index->tableQname);
 	}
 
 	if (!copydb_index_is_being_processed(index,
@@ -767,6 +768,14 @@ copydb_create_index(const char *pguri,
 
 	if (!skipCreateIndex)
 	{
+		char psTitle[BUFSIZE] = { 0 };
+
+		sformat(psTitle, sizeof(psTitle), "pgcopydb: create index %s.%s",
+				index->indexNamespace,
+				index->indexRelname);
+
+		(void) set_ps_title(psTitle);
+
 		PGSQL dst = { 0 };
 
 		log_notice("%s", summary->command);
@@ -1007,6 +1016,7 @@ copydb_prepare_create_index_command(SourceIndex *index,
 		else
 		{
 			log_error("Failed to parse \"%s\"", index->indexDef);
+			destroyPQExpBuffer(cmd);
 			return false;
 		}
 	}
@@ -1049,11 +1059,10 @@ copydb_prepare_create_constraint_command(SourceIndex *index, char **command)
 		char *constraintType = index->isPrimary ? "PRIMARY KEY" : "UNIQUE";
 
 		appendPQExpBuffer(cmd,
-						  "ALTER TABLE \"%s\".\"%s\" "
-						  "ADD CONSTRAINT \"%s\" %s "
-						  "USING INDEX \"%s\"",
-						  index->tableNamespace,
-						  index->tableRelname,
+						  "ALTER TABLE %s "
+						  "ADD CONSTRAINT %s %s "
+						  "USING INDEX %s",
+						  index->tableQname,
 						  index->constraintName,
 						  constraintType,
 						  index->indexRelname);
@@ -1061,10 +1070,9 @@ copydb_prepare_create_constraint_command(SourceIndex *index, char **command)
 	else
 	{
 		appendPQExpBuffer(cmd,
-						  "ALTER TABLE \"%s\".\"%s\" "
-						  "ADD CONSTRAINT \"%s\" %s ",
-						  index->tableNamespace,
-						  index->tableRelname,
+						  "ALTER TABLE %s "
+						  "ADD CONSTRAINT %s %s ",
+						  index->tableQname,
 						  index->constraintName,
 						  index->constraintDef);
 	}
@@ -1154,10 +1162,9 @@ copydb_create_constraints(CopyDataSpec *specs, SourceTable *table)
 			specs->section == DATA_SECTION_ALL ? LOG_NOTICE : LOG_INFO;
 
 		log_level(logLevel,
-				  "Found %d indexes on target database for table \"%s\".\"%s\"",
+				  "Found %d indexes on target database for table %s",
 				  dstIndexArray.count,
-				  table->nspname,
-				  table->relname);
+				  table->qname);
 	}
 
 	SourceIndexList *indexListEntry = table->firstIndex;
