@@ -225,8 +225,10 @@ stream_init_for_mode(StreamSpecs *specs, LogicalStreamMode mode)
 	}
 	else if (specs->mode == STREAM_MODE_REPLAY && mode == STREAM_MODE_CATCHUP)
 	{
+		specs->stdIn = false;
+		specs->stdOut = false;
+
 		/* we keep the transform queue around */
-		(void) 0;
 	}
 	else
 	{
@@ -286,6 +288,46 @@ LogicalStreamModeToString(LogicalStreamMode mode)
 }
 
 
+/*
+ * stream_check_in_out checks that the stdIn and stdOut file descriptors are
+ * still valid: EBADF could happen when a PIPE is Broken for lack of a
+ * reader/writer process.
+ */
+bool
+stream_check_in_out(StreamSpecs *specs)
+{
+	if (specs->stdIn)
+	{
+		char buf[0];
+
+		if (read(fileno(specs->in), buf, 0) != 0)
+		{
+			log_error("Failed to read from input PIPE: %m");
+			return false;
+		}
+	}
+
+	if (specs->stdOut)
+	{
+		char buf[0];
+
+		if (fwrite(buf, sizeof(char), 0, specs->in) != 0)
+		{
+			log_error("Failed to write to output PIPE: %m");
+			return false;
+		}
+
+		if (fflush(specs->out) != 0)
+		{
+			log_error("Failed to flush output PIPE: %m");
+			return false;
+		}
+	}
+
+	return true;
+}
+
+
 /*
  * stream_init_context initializes a LogicalStreamContext.
  */
@@ -299,11 +341,6 @@ stream_init_context(StreamSpecs *specs)
 	privateContext->startposActionFromJSON = specs->startposActionFromJSON;
 
 	privateContext->mode = specs->mode;
-	privateContext->stdIn = specs->stdIn;
-	privateContext->stdOut = specs->stdOut;
-
-	privateContext->in = specs->in;
-	privateContext->out = specs->out;
 
 	privateContext->transformQueue = &(specs->transformQueue);
 
@@ -312,10 +349,21 @@ stream_init_context(StreamSpecs *specs)
 
 	privateContext->connStrings = specs->connStrings;
 
-	privateContext->metadata.action = STREAM_ACTION_UNKNOWN;
-	privateContext->previous.action = STREAM_ACTION_UNKNOWN;
+	/*
+	 * When using PIPEs for inter-process communication, makes sure the PIPEs
+	 * are ready for us to use and not broken, as in EBADF.
+	 */
+	privateContext->stdIn = specs->stdIn;
+	privateContext->stdOut = specs->stdOut;
 
-	privateContext->lastWriteTime = 0;
+	privateContext->in = specs->in;
+	privateContext->out = specs->out;
+
+	if (!stream_check_in_out(specs))
+	{
+		/* errors have already been logged */
+		return false;
+	}
 
 	/*
 	 * When streaming is resumed, transactions are sent in full even if we wrote
@@ -326,6 +374,11 @@ stream_init_context(StreamSpecs *specs)
 	 * However, note that if the last message is COMMIT, the streaming will
 	 * resume from the next transaction.
 	 */
+	privateContext->metadata.action = STREAM_ACTION_UNKNOWN;
+	privateContext->previous.action = STREAM_ACTION_UNKNOWN;
+
+	privateContext->lastWriteTime = 0;
+
 	if (specs->startposComputedFromJSON &&
 		(specs->startposActionFromJSON == STREAM_ACTION_BEGIN ||
 		 specs->startposActionFromJSON == STREAM_ACTION_INSERT ||
@@ -404,16 +457,6 @@ startLogicalStreaming(StreamSpecs *specs)
 	StreamContext *privateContext = &(specs->private);
 	context.private = (void *) privateContext;
 
-	if (specs->stdOut)
-	{
-		/* switch stdout from block buffered to line buffered mode */
-		if (setvbuf(specs->out, NULL, _IOLBF, 0) != 0)
-		{
-			log_error("Failed to set stdout to line buffered mode: %m");
-			return false;
-		}
-	}
-
 	log_notice("Connecting to logical decoding replication stream");
 
 	/*
@@ -421,9 +464,23 @@ startLogicalStreaming(StreamSpecs *specs)
 	 * continue streaming.
 	 */
 	bool retry = true;
+	uint64_t retries = 0;
+	uint64_t waterMarkLSN = InvalidXLogRecPtr;
 
 	while (retry)
 	{
+		if (!stream_check_in_out(specs))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+
+		if (asked_to_stop || asked_to_stop_fast || asked_to_quit)
+		{
+			log_error("Streaming process has been signaled to stop");
+			return false;
+		}
+
 		if (!pgsql_init_stream(&stream,
 							   specs->connStrings->logrep_pguri,
 							   specs->slot.plugin,
@@ -462,11 +519,22 @@ startLogicalStreaming(StreamSpecs *specs)
 
 		if (cleanExit)
 		{
-			log_info("Streamed up to write_lsn %X/%X, flush_lsn %X/%X, stopping",
+			log_info("Streamed up to write_lsn %X/%X, flush_lsn %X/%X, stopping: "
+					 "endpos is %X/%X",
 					 LSN_FORMAT_ARGS(context.tracking->written_lsn),
-					 LSN_FORMAT_ARGS(context.tracking->flushed_lsn));
+					 LSN_FORMAT_ARGS(context.tracking->flushed_lsn),
+					 LSN_FORMAT_ARGS(context.endpos));
 		}
-		else if (!(asked_to_stop || asked_to_stop_fast || asked_to_quit))
+		else if (retries > 0 &&
+				 context.tracking->written_lsn == waterMarkLSN)
+		{
+			log_warn("Streaming got interrupted at %X/%X, and did not make "
+					 "any progress from previous attempt, stopping now",
+					 LSN_FORMAT_ARGS(context.tracking->written_lsn));
+
+			return false;
+		}
+		else if (retry)
 		{
 			log_warn("Streaming got interrupted at %X/%X, reconnecting in 1s",
 					 LSN_FORMAT_ARGS(context.tracking->written_lsn));
@@ -483,7 +551,10 @@ startLogicalStreaming(StreamSpecs *specs)
 		/* sleep for one entire second before retrying */
 		if (retry)
 		{
-			(void) pg_usleep(1 * 1000 * 1000);
+			++retries;
+			waterMarkLSN = context.tracking->written_lsn;
+
+			(void) pg_usleep(1 * 1000 * 1000); /* 1s */
 		}
 	}
 
@@ -2055,6 +2126,7 @@ buildReplicationURI(const char *pguri, char **repl_pguri)
 
 	/* if replication is already found, we override it to value "1" */
 	if (!parse_pguri_info_key_vals(pguri,
+								   &connStringDefaults,
 								   &replicationParams,
 								   &params,
 								   checkForCompleteURI))
@@ -2474,7 +2546,7 @@ stream_create_sentinel(CopyDataSpec *copySpecs,
 		if (!pgsql_execute(pgsql, sql[i]))
 		{
 			/* errors have already been logged */
-			exit(EXIT_CODE_SOURCE);
+			return false;
 		}
 	}
 
@@ -2599,6 +2671,9 @@ stream_read_context(CDCPaths *paths,
 					uint32_t *WalSegSz)
 {
 	char *wal_segment_size = NULL;
+	char *tli = NULL;
+	char *history = NULL;
+
 	long size = 0L;
 
 	/*
@@ -2610,7 +2685,7 @@ stream_read_context(CDCPaths *paths,
 	ConnectionRetryPolicy retryPolicy = { 0 };
 
 	int maxT = 10;              /* 10s */
-	int maxSleepTime = 1000;    /* 1s */
+	int maxSleepTime = 1500;    /* 1.5s */
 	int baseSleepTime = 100;    /* 100ms */
 
 	(void) pgsql_set_retry_policy(&retryPolicy,
@@ -2621,12 +2696,35 @@ stream_read_context(CDCPaths *paths,
 
 	while (!pgsql_retry_policy_expired(&retryPolicy))
 	{
+		if (asked_to_stop || asked_to_stop_fast || asked_to_quit)
+		{
+			return false;
+		}
+
 		if (file_exists(paths->walsegsizefile) &&
 			file_exists(paths->tlifile) &&
 			file_exists(paths->tlihistfile))
 		{
-			/* success: break out of the retry loop */
-			break;
+			/*
+			 * File did exist, but might have been deleted now (race condition
+			 * at prefetch and transform processes start-up).
+			 */
+			bool success = true;
+
+			success = success &&
+					  read_file(paths->walsegsizefile, &wal_segment_size, &size);
+
+			success = success &&
+					  read_file(paths->tlifile, &tli, &size);
+
+			success = success &&
+					  read_file(paths->tlihistfile, &history, &size);
+
+			if (success)
+			{
+				/* success: break out of the retry loop */
+				break;
+			}
 		}
 
 		int sleepTimeMs =
@@ -2640,46 +2738,48 @@ stream_read_context(CDCPaths *paths,
 		(void) pg_usleep(sleepTimeMs * 1000);
 	}
 
-	/* we don't want to retry anymore, error out if files still don't exist */
-	if (!read_file(paths->walsegsizefile, &wal_segment_size, &size))
+	/* did retry policy expire before the files are created? */
+	if (!(file_exists(paths->walsegsizefile) &&
+		  file_exists(paths->tlifile) &&
+		  file_exists(paths->tlihistfile)))
 	{
-		/* errors have already been logged */
+		log_error("Failed to read stream context file: retry policy expired");
 		return false;
 	}
 
+	/*
+	 * Now that we could read the file contents, parse it.
+	 */
 	if (!stringToUInt(wal_segment_size, WalSegSz))
 	{
 		/* errors have already been logged */
-		return false;
-	}
-
-	char *tli;
-
-	if (!read_file(paths->tlifile, &tli, &size))
-	{
-		/* errors have already been logged */
+		free(wal_segment_size);
+		free(tli);
+		free(history);
 		return false;
 	}
 
 	if (!stringToUInt(tli, &(system->timeline)))
 	{
 		/* errors have already been logged */
-		return false;
-	}
-
-	char *history = NULL;
-
-	if (!read_file(paths->tlihistfile, &history, &size))
-	{
-		/* errors have already been logged */
+		free(wal_segment_size);
+		free(tli);
+		free(history);
 		return false;
 	}
 
 	if (!parseTimeLineHistory(paths->tlihistfile, history, system))
 	{
 		/* errors have already been logged */
+		free(wal_segment_size);
+		free(tli);
+		free(history);
 		return false;
 	}
 
+	free(wal_segment_size);
+	free(tli);
+	free(history);
+
 	return true;
 }
