@@ -18,6 +18,7 @@
 #include "libpq-fe.h"
 #include "pqexpbuffer.h"
 
+#include "copydb.h"
 #include "defaults.h"
 #include "log.h"
 #include "parsing_utils.h"
@@ -528,6 +529,7 @@ parse_pretty_printed_bytes(const char *value, uint64_t *result)
  */
 bool
 parse_pguri_info_key_vals(const char *pguri,
+						  KeyVal *defaults,
 						  KeyVal *overrides,
 						  URIParams *uriParameters,
 						  bool checkForCompleteURI)
@@ -553,7 +555,6 @@ parse_pguri_info_key_vals(const char *pguri,
 	for (option = conninfo; option->keyword != NULL; option++)
 	{
 		char *value = NULL;
-		int ovIndex = 0;
 
 		/*
 		 * If the keyword is in our overrides array, use the value from the
@@ -561,18 +562,18 @@ parse_pguri_info_key_vals(const char *pguri,
 		 * something very small, like 3 (typically: sslmode, sslrootcert,
 		 * sslcrl).
 		 */
-		for (ovIndex = 0; ovIndex < overrides->count; ovIndex++)
+		for (int ovIndex = 0; ovIndex < overrides->count; ovIndex++)
 		{
-			if (strcmp(overrides->keywords[ovIndex], option->keyword) == 0)
+			if (streq(overrides->keywords[ovIndex], option->keyword))
 			{
 				value = overrides->values[ovIndex];
 			}
 		}
 
-		/* not found in the override, keep the original, or skip */
+		/* now either take the given value or maybe skip the keyword */
 		if (value == NULL)
 		{
-			if (option->val == NULL || strcmp(option->val, "") == 0)
+			if (option->val == NULL || streq(option->val, ""))
 			{
 				continue;
 			}
@@ -603,7 +604,7 @@ parse_pguri_info_key_vals(const char *pguri,
 			foundDBName = true;
 			uriParameters->dbname = strdup(option->val);
 		}
-		else if (!IS_EMPTY_STRING_BUFFER(value))
+		else if (value != NULL && !streq(value, ""))
 		{
 			/* make a copy in our key/val arrays */
 			uriParameters->parameters.keywords[paramIndex] =
@@ -617,6 +618,37 @@ parse_pguri_info_key_vals(const char *pguri,
 		}
 	}
 
+	/*
+	 * Now add-in the default values that we have, unless they have been
+	 * provided in the previous round.
+	 */
+	for (int defIndex = 0; defIndex < defaults->count; defIndex++)
+	{
+		char *keyword = defaults->keywords[defIndex];
+		char *value = defaults->values[defIndex];
+
+		bool found = false;
+
+		for (option = conninfo; option->keyword != NULL; option++)
+		{
+			if (streq(keyword, option->keyword))
+			{
+				found = option->val != NULL && !streq(option->val, "");
+				break;
+			}
+		}
+
+		if (!found)
+		{
+			/* make a copy in our key/val arrays */
+			uriParameters->parameters.keywords[paramIndex] = strdup(keyword);
+			uriParameters->parameters.values[paramIndex] = strdup(value);
+
+			++uriParameters->parameters.count;
+			++paramIndex;
+		}
+	}
+
 	PQconninfoFree(conninfo);
 
 	/*
@@ -711,6 +743,7 @@ buildPostgresURIfromPieces(URIParams *uriParams, char **pguri)
 		{
 			log_error("Failed to percent-escape URI dbname \"%s\"",
 					  uriParams->dbname);
+			destroyPQExpBuffer(uri);
 			return false;
 		}
 		appendPQExpBuffer(uri, "%s", escaped);
@@ -903,9 +936,9 @@ escapeWithPercentEncoding(const char *str, char **dst)
 
 
 /*
- * uri_contains_password takes a Postgres connection string and checks to see
- * if it contains a parameter called password. Returns true if a password
- * keyword is present in the connection string.
+ * uri_grab_password takes a Postgres connection string and checks to see
+ * if it contains a parameter called password and if so stores a copy of it
+ * in safeURI->password.
  */
 static bool
 uri_grab_password(const char *pguri, SafeURI *safeURI)
@@ -951,8 +984,8 @@ uri_grab_password(const char *pguri, SafeURI *safeURI)
 
 /*
  * parse_and_scrub_connection_string takes a Postgres connection string and
- * populates scrubbedPguri with the password replaced with **** for logging.
- * The scrubbedPguri parameter should point to a memory area that has been
+ * populates safeURI without the password for logging purposes.
+ * The safeURI parameter should point to a memory area that has been
  * allocated by the caller and has at least MAXCONNINFO bytes.
  */
 bool
@@ -981,6 +1014,7 @@ parse_and_scrub_connection_string(const char *pguri, SafeURI *safeURI)
 	bool checkForCompleteURI = false;
 
 	if (!parse_pguri_info_key_vals(pguri,
+								   &connStringDefaults,
 								   &overrides,
 								   uriParams,
 								   checkForCompleteURI))
@@ -1045,4 +1079,4 @@ freeKeyVal(KeyVal *parameters)
 	}
 
 	parameters->count = 0;
-}
+}
\ No newline at end of file
