@@ -46,7 +46,7 @@
 /*
  * Chunk size for reading and writting large objects
  */
-#define LOBBUFSIZE 16384
+#define LOBBUFSIZE 16 * 1024 * 1024 /* 16 MB */
 
 
 /*
@@ -235,7 +235,6 @@ typedef struct GUC
 	char *value;
 } GUC;
 
-
 bool pgsql_init(PGSQL *pgsql, char *url, ConnectionType connectionType);
 
 void pgsql_set_retry_policy(ConnectionRetryPolicy *retryPolicy,
@@ -243,7 +242,6 @@ void pgsql_set_retry_policy(ConnectionRetryPolicy *retryPolicy,
 							int maxR,
 							int maxSleepTime,
 							int baseSleepTime);
-void pgsql_set_main_loop_retry_policy(ConnectionRetryPolicy *retryPolicy);
 void pgsql_set_interactive_retry_policy(ConnectionRetryPolicy *retryPolicy);
 int pgsql_compute_connection_retry_sleep_time(ConnectionRetryPolicy *retryPolicy);
 bool pgsql_retry_policy_expired(ConnectionRetryPolicy *retryPolicy);
@@ -290,6 +288,19 @@ bool pgsql_execute_with_params(PGSQL *pgsql, const char *sql, int paramCount,
 							   const Oid *paramTypes, const char **paramValues,
 							   void *parseContext, ParsePostgresResultCB *parseFun);
 
+bool pgsql_send_with_params(PGSQL *pgsql, const char *sql, int paramCount,
+							const Oid *paramTypes, const char **paramValues);
+
+bool pgsql_fetch_results(PGSQL *pgsql, bool *done,
+						 void *context, ParsePostgresResultCB *parseFun);
+
+bool pgsql_prepare(PGSQL *pgsql, const char *name, const char *sql,
+				   int paramCount, const Oid *paramTypes);
+
+bool pgsql_execute_prepared(PGSQL *pgsql, const char *name,
+							int paramCount, const char **paramValues,
+							void *context, ParsePostgresResultCB *parseFun);
+
 void pgAutoCtlDebugNoticeProcessor(void *arg, const char *message);
 
 bool validate_connection_string(const char *connectionString);
@@ -309,8 +320,10 @@ bool pgsql_get_sequence(PGSQL *pgsql, const char *nspname, const char *relname,
 
 bool pgsql_set_gucs(PGSQL *pgsql, GUC *settings);
 
-bool pg_copy_large_objects(PGSQL *src, PGSQL *dst,
-						   bool dropIfExists, uint32_t *count);
+bool pg_copy_large_object(PGSQL *src,
+						  PGSQL *dst,
+						  bool dropIfExists,
+						  uint32_t oid);
 
 /*
  * Maximum length of serialized pg_lsn value
@@ -513,6 +526,8 @@ bool pgsql_table_exists(PGSQL *pgsql,
 						const char *nspname,
 						bool *exists);
 
+bool pgsql_current_wal_flush_lsn(PGSQL *pgsql, uint64_t *lsn);
+bool pgsql_current_wal_insert_lsn(PGSQL *pgsql, uint64_t *lsn);
 
 /*
  * pgcopydb sentinel is a table that's created on the source database and
