@@ -13,9 +13,7 @@
 
 #include "filtering.h"
 #include "pgsql.h"
-
-/* the pg_restore -l output uses "schema name owner" */
-#define RESTORE_LIST_NAMEDATALEN (3 * NAMEDATALEN + 3)
+#include "pg_utils.h"
 
 /*
  * In the SQL standard we have "catalogs", which are then Postgres databases.
@@ -24,9 +22,9 @@
 typedef struct SourceDatabase
 {
 	uint32_t oid;
-	char datname[NAMEDATALEN];
+	char datname[PG_NAMEDATALEN];
 	int64_t bytes;
-	char bytesPretty[NAMEDATALEN]; /* pg_size_pretty */
+	char bytesPretty[PG_NAMEDATALEN]; /* pg_size_pretty */
 }
 SourceDatabase;
 
@@ -37,10 +35,26 @@ typedef struct SourceDatabaseArray
 } SourceDatabaseArray;
 
 
+typedef struct SourceRole
+{
+	uint32_t oid;
+	char rolname[PG_NAMEDATALEN];
+
+	UT_hash_handle hh;          /* makes this structure hashable */
+} SourceRole;
+
+
+typedef struct SourceRoleArray
+{
+	int count;
+	SourceRole *array;          /* malloc'ed area */
+} SourceRoleArray;
+
+
 typedef struct SourceSchema
 {
 	uint32_t oid;
-	char nspname[NAMEDATALEN];
+	char nspname[PG_NAMEDATALEN];
 	char restoreListName[RESTORE_LIST_NAMEDATALEN];
 
 	UT_hash_handle hh;          /* makes this structure hashable */
@@ -60,8 +74,8 @@ typedef struct SourceSchemaArray
 typedef struct SourceExtensionConfig
 {
 	uint32_t oid;               /* pg_class.oid */
-	char nspname[NAMEDATALEN];
-	char relname[NAMEDATALEN];
+	char nspname[PG_NAMEDATALEN];
+	char relname[PG_NAMEDATALEN];
 	char *condition;            /* strdup from PQresult: malloc'ed area */
 } SourceExtensionConfig;
 
@@ -76,8 +90,8 @@ typedef struct SourceExtensionConfigArray
 typedef struct SourceExtension
 {
 	uint32_t oid;
-	char extname[NAMEDATALEN];
-	char extnamespace[NAMEDATALEN];
+	char extname[PG_NAMEDATALEN];
+	char extnamespace[PG_NAMEDATALEN];
 	bool extrelocatable;
 	SourceExtensionConfigArray config;
 } SourceExtension;
@@ -92,7 +106,7 @@ typedef struct SourceExtensionArray
 
 typedef struct ExtensionsVersions
 {
-	char name[NAMEDATALEN];
+	char name[PG_NAMEDATALEN];
 	char defaultVersion[BUFSIZE];
 	char installedVersion[BUFSIZE];
 	JSON_Value *json;           /* malloc'ed area */
@@ -107,7 +121,7 @@ typedef struct ExtensionsVersionsArray
 typedef struct SourceCollation
 {
 	uint32_t oid;
-	char collname[NAMEDATALEN];
+	char collname[PG_NAMEDATALEN];
 	char *desc;                 /* malloc'ed area */
 	char restoreListName[RESTORE_LIST_NAMEDATALEN];
 } SourceCollation;
@@ -146,8 +160,9 @@ typedef struct SourceTableAttribute
 {
 	int attnum;
 	uint32_t atttypid;
-	char attname[NAMEDATALEN];
+	char attname[PG_NAMEDATALEN];
 	bool attisprimary;
+	bool attisgenerated;
 } SourceTableAttribute;
 
 typedef struct SourceTableAttributeArray
@@ -159,24 +174,35 @@ typedef struct SourceTableAttributeArray
 /* forward declaration */
 struct SourceIndexList;
 
+/* checksum is formatted as uuid */
+#define CHECKSUMLEN 36
+
+typedef struct TableChecksum
+{
+	uint64_t rowcount;
+	char checksum[CHECKSUMLEN];
+} TableChecksum;
+
 typedef struct SourceTable
 {
 	uint32_t oid;
 
-	/* "nspname"."relname" : 4 quotes, 1 dot, 1 \0 */
-	char qname[NAMEDATALEN * 2 + 5 + 1];
-	char nspname[NAMEDATALEN];
-	char relname[NAMEDATALEN];
+	char qname[PG_NAMEDATALEN_FQ];
+	char nspname[PG_NAMEDATALEN];
+	char relname[PG_NAMEDATALEN];
+	char amname[PG_NAMEDATALEN];
+
+	int64_t relpages;
 	int64_t reltuples;
 	int64_t bytes;
-	char bytesPretty[NAMEDATALEN]; /* pg_size_pretty */
+	char bytesPretty[PG_NAMEDATALEN]; /* pg_size_pretty */
 	bool excludeData;
 
-	uint64_t rowcount;
-	uint64_t checksum;
+	TableChecksum sourceChecksum;
+	TableChecksum targetChecksum;
 
 	char restoreListName[RESTORE_LIST_NAMEDATALEN];
-	char partKey[NAMEDATALEN];
+	char partKey[PG_NAMEDATALEN];
 	SourceTablePartsArray partsArray;
 
 	SourceTableAttributeArray attributes;
@@ -207,9 +233,9 @@ typedef struct SourceSequence
 	uint32_t attrelid;          /* pg_class oid of table using as DEFAULT */
 	uint32_t attroid;           /* pg_attrdef DEFAULT value OID */
 
-	char qname[NAMEDATALEN * 2 + 5 + 1];
-	char nspname[NAMEDATALEN];
-	char relname[NAMEDATALEN];
+	char qname[PG_NAMEDATALEN_FQ];
+	char nspname[PG_NAMEDATALEN];
+	char relname[PG_NAMEDATALEN];
 	int64_t lastValue;
 	bool isCalled;
 
@@ -233,20 +259,26 @@ typedef struct SourceSequenceArray
 typedef struct SourceIndex
 {
 	uint32_t indexOid;
-	char indexNamespace[NAMEDATALEN];
-	char indexRelname[NAMEDATALEN];
+	char indexQname[PG_NAMEDATALEN_FQ];
+	char indexNamespace[PG_NAMEDATALEN];
+	char indexRelname[PG_NAMEDATALEN];
+
 	uint32_t tableOid;
-	char tableNamespace[NAMEDATALEN];
-	char tableRelname[NAMEDATALEN];
+	char tableQname[PG_NAMEDATALEN_FQ];
+	char tableNamespace[PG_NAMEDATALEN];
+	char tableRelname[PG_NAMEDATALEN];
+
 	bool isPrimary;
 	bool isUnique;
 	char *indexColumns;         /* malloc'ed area */
 	char *indexDef;             /* malloc'ed area */
+
 	uint32_t constraintOid;
 	bool condeferrable;
 	bool condeferred;
-	char constraintName[NAMEDATALEN];
+	char constraintName[PG_NAMEDATALEN];
 	char *constraintDef;        /* malloc'ed area */
+
 	char indexRestoreListName[RESTORE_LIST_NAMEDATALEN];
 	char constraintRestoreListName[RESTORE_LIST_NAMEDATALEN];
 
@@ -276,8 +308,8 @@ typedef struct SourceIndexList
  */
 typedef struct SourceDepend
 {
-	char nspname[NAMEDATALEN];
-	char relname[NAMEDATALEN];
+	char nspname[PG_NAMEDATALEN];
+	char relname[PG_NAMEDATALEN];
 	uint32_t refclassid;
 	uint32_t refobjid;
 	uint32_t classid;
@@ -294,12 +326,38 @@ typedef struct SourceDependArray
 	SourceDepend *array;         /* malloc'ed area */
 } SourceDependArray;
 
+
+/*
+ * SourceProperty caches data found in Postgres catalog pg_db_role_setting,
+ * allowing to support ALTER DATABASE SET and ALTER ROLE IN DATABASE
+ * properties.
+ *
+ * The setconfig format ("name=value") from the catalogs needs specific parsing
+ * and re-writting in order to create the SQL statement needed to re-install
+ * the properties, this is done when applying the properties, the same way as
+ * pg_dump.
+ */
+typedef struct SourceProperty
+{
+	bool roleInDatabase;
+	char rolname[PG_NAMEDATALEN];
+	char datname[PG_NAMEDATALEN];
+	char *setconfig;            /* malloc'ed area */
+} SourceProperty;
+
+typedef struct SourcePropertiesArray
+{
+	int count;
+	SourceProperty *array;      /* malloc'ed area */
+} SourcePropertiesArray;
+
 /*
  * SourceCatalog regroups all the information we fetch from a Postgres
  * instance.
  */
 typedef struct SourceCatalog
 {
+	SourcePropertiesArray gucsArray;
 	SourceExtensionArray extensionArray;
 	SourceCollationArray collationArray;
 	SourceTableArray sourceTableArray;
@@ -316,7 +374,10 @@ typedef struct SourceCatalog
 
 typedef struct TargetCatalog
 {
+	SourceRoleArray rolesArray;
 	SourceSchemaArray schemaArray;
+
+	SourceRole *rolesHashByName;
 	SourceSchema *schemaHashByName;
 } TargetCatalog;
 
@@ -327,8 +388,13 @@ bool schema_query_privileges(PGSQL *pgsql,
 
 bool schema_list_databases(PGSQL *pgsql, SourceDatabaseArray *catArray);
 
+bool schema_list_database_properties(PGSQL *pgsql,
+									 SourcePropertiesArray *gucsArray);
+
 bool schema_list_schemas(PGSQL *pgsql, SourceSchemaArray *array);
 
+bool schema_list_roles(PGSQL *pgsql, SourceRoleArray *rolesArray);
+
 bool schema_list_ext_schemas(PGSQL *pgsql, SourceSchemaArray *array);
 
 bool schema_list_extensions(PGSQL *pgsql, SourceExtensionArray *extArray);
@@ -376,6 +442,7 @@ bool schema_list_pg_depend(PGSQL *pgsql,
 						   SourceFilters *filters,
 						   SourceDependArray *dependArray);
 
-bool schema_checksum_table(PGSQL *pgsql, SourceTable *table);
+bool schema_send_table_checksum(PGSQL *pgsql, SourceTable *table);
+bool schema_fetch_table_checksum(PGSQL *pgsql, TableChecksum *sum, bool *done);
 
 #endif /* SCHEMA_H */
