@@ -117,16 +117,16 @@ copydb_process_table_data(CopyDataSpec *specs)
 		if (!copydb_start_copy_supervisor(specs))
 		{
 			/* errors have already been logged */
-			return false;
+			++errors;
 		}
 
 		/*
 		 * Start as many index worker process as --index-jobs
 		 */
-		if (!copydb_start_index_workers(specs))
+		if (errors == 0 && !copydb_start_index_workers(specs))
 		{
 			/* errors have already been logged */
-			return false;
+			++errors;
 		}
 
 		/*
@@ -134,10 +134,10 @@ copydb_process_table_data(CopyDataSpec *specs)
 		 * --table-jobs. Could be exposed separately as --vacuumJobs too, but
 		 * that's not been done at this time.
 		 */
-		if (!vacuum_start_workers(specs))
+		if (errors == 0 && !vacuum_start_workers(specs))
 		{
 			/* errors have already been logged */
-			return false;
+			++errors;
 		}
 	}
 	else
@@ -151,19 +151,35 @@ copydb_process_table_data(CopyDataSpec *specs)
 	/*
 	 * Are blobs table data? well pg_dump --section sayth yes.
 	 */
-	if (!copydb_start_blob_process(specs))
+	if (errors == 0 && !copydb_start_blob_process(specs))
 	{
 		/* errors have already been logged */
-		return false;
+		++errors;
 	}
 
 	/*
 	 * Start an auxilliary process to reset sequences on the target database.
 	 */
-	if (!copydb_start_seq_process(specs))
+	if (errors == 0 && !copydb_start_seq_process(specs))
 	{
 		/* errors have already been logged */
-		return false;
+		++errors;
+	}
+
+	/* when errors happened, signal all processes to terminate now */
+	if (errors > 0)
+	{
+		log_error("Failed to start some worker processes, "
+				  "see above for details");
+
+		/* send TERM signal to all the process in our process group */
+		if (!kill(0, SIGTERM))
+		{
+			log_error("Failed to send TERM signal our process group");
+
+			/* refrain from early return here, we want to waitpid() */
+			++errors;
+		}
 	}
 
 	if (!copydb_wait_for_subprocesses(specs->failFast))
@@ -204,13 +220,15 @@ copydb_start_copy_supervisor(CopyDataSpec *specs)
 	{
 		case -1:
 		{
-			log_error("Failed to fork a worker process: %m");
+			log_error("Failed to fork copy supervisor process: %m");
 			return false;
 		}
 
 		case 0:
 		{
 			/* child process runs the command */
+			(void) set_ps_title("pgcopydb: copy supervisor");
+
 			if (!copydb_copy_supervisor(specs))
 			{
 				log_error("Failed to copy table data, see above for details");
@@ -233,7 +251,7 @@ copydb_start_copy_supervisor(CopyDataSpec *specs)
 
 
 /*
- * copydb_start_copy_supervisor creates the copyQueue and if needed the
+ * copydb_copy_supervisor creates the copyQueue and if needed the
  * copyDoneQueue too, then starts --table-jobs COPY table data workers to
  * process table oids from the queue.
  */
@@ -255,7 +273,11 @@ copydb_copy_supervisor(CopyDataSpec *specs)
 	 */
 	if (!copydb_start_table_data_workers(specs))
 	{
-		/* errors have already been logged */
+		log_fatal("Failed to start table data COPY workers, "
+				  "see above for details");
+
+		(void) copydb_fatal_exit();
+
 		return false;
 	}
 
@@ -273,6 +295,8 @@ copydb_copy_supervisor(CopyDataSpec *specs)
 		if (!copydb_add_copy(specs, table->oid, tableSpecs->part.partNumber))
 		{
 			/* errors have already been logged */
+			(void) copydb_fatal_exit();
+
 			return false;
 		}
 	}
@@ -280,15 +304,14 @@ copydb_copy_supervisor(CopyDataSpec *specs)
 	/*
 	 * Add the STOP messages to the queue now, one STOP message per worker.
 	 */
-	for (int i = 0; i < specs->tableJobs; i++)
+	if (!copydb_copy_supervisor_send_stop(specs))
 	{
-		QMessage stop = { .type = QMSG_TYPE_STOP };
+		log_fatal("Failed to send STOP messages to the COPY queue");
 
-		if (!queue_send(&(specs->copyQueue), &stop))
-		{
-			/* errors have already been logged */
-			return false;
-		}
+		/* we still need to make sure the COPY processes terminate */
+		(void) copydb_fatal_exit();
+
+		return false;
 	}
 
 	/*
@@ -298,6 +321,14 @@ copydb_copy_supervisor(CopyDataSpec *specs)
 	{
 		log_error("Some COPY worker process(es) have exited with error, "
 				  "see above for details");
+
+		/* make sure vacuum and create index processes see a STOP message */
+		if (!vacuum_send_stop(specs) ||
+			!copydb_index_workers_send_stop(specs))
+		{
+			(void) copydb_fatal_exit();
+		}
+
 		return false;
 	}
 
@@ -318,7 +349,42 @@ copydb_copy_supervisor(CopyDataSpec *specs)
 	success = success && vacuum_send_stop(specs);
 	success = success && copydb_index_workers_send_stop(specs);
 
-	return success;
+	if (!success)
+	{
+		/*
+		 * The other subprocesses need to see a STOP message to stop their
+		 * processing. Failing to send the STOP messages means that the main
+		 * pgcopydb never finishes, and we want to ensure the command
+		 * terminates.
+		 */
+		(void) copydb_fatal_exit();
+
+		return false;
+	}
+
+	return true;
+}
+
+
+/*
+ * copydb_copy_supervisor_send_stop sends the STOP messages to the copy queue,
+ * one STOP message per worker.
+ */
+bool
+copydb_copy_supervisor_send_stop(CopyDataSpec *specs)
+{
+	for (int i = 0; i < specs->tableJobs; i++)
+	{
+		QMessage stop = { .type = QMSG_TYPE_STOP };
+
+		if (!queue_send(&(specs->copyQueue), &stop))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+	}
+
+	return true;
 }
 
 
@@ -369,12 +435,14 @@ copydb_start_table_data_workers(CopyDataSpec *specs)
 			case -1:
 			{
 				log_error("Failed to fork a COPY worker process: %m");
-				exit(EXIT_CODE_INTERNAL_ERROR);
+				return false;
 			}
 
 			case 0:
 			{
 				/* child process runs the command */
+				(void) set_ps_title("pgcopydb: copy worker");
+
 				if (!copydb_table_data_worker(specs))
 				{
 					/* errors have already been logged */
@@ -404,6 +472,7 @@ copydb_start_table_data_workers(CopyDataSpec *specs)
 bool
 copydb_table_data_worker(CopyDataSpec *specs)
 {
+	uint64_t errors = 0;
 	pid_t pid = getpid();
 
 	log_notice("Started table data COPY worker %d [%d]", pid, getppid());
@@ -417,8 +486,8 @@ copydb_table_data_worker(CopyDataSpec *specs)
 
 	while (true)
 	{
-		QMessage mesg = { 0 };
-		bool recv_ok = queue_receive(&(specs->copyQueue), &mesg);
+		QMessage *mesg = (QMessage *) calloc(1, sizeof(QMessage));
+		bool recv_ok = queue_receive(&(specs->copyQueue), mesg);
 
 		if (asked_to_stop || asked_to_stop_fast || asked_to_quit)
 		{
@@ -432,26 +501,44 @@ copydb_table_data_worker(CopyDataSpec *specs)
 			break;
 		}
 
-		switch (mesg.type)
+		switch (mesg->type)
 		{
 			case QMSG_TYPE_STOP:
 			{
 				log_debug("Stop message received by COPY worker");
 				(void) copydb_close_snapshot(specs);
+				free(mesg);
 				return true;
 			}
 
 			case QMSG_TYPE_TABLEPOID:
 			{
 				if (!copydb_copy_data_by_oid(specs,
-											 mesg.data.tp.oid,
-											 mesg.data.tp.part))
+											 mesg->data.tp.oid,
+											 mesg->data.tp.part))
 				{
 					log_error("Failed to copy data for table with oid %u "
 							  "and part number %u, see above for details",
-							  mesg.data.tp.oid,
-							  mesg.data.tp.part);
-					return false;
+							  mesg->data.tp.oid,
+							  mesg->data.tp.part);
+
+					++errors;
+
+					if (specs->failFast)
+					{
+						free(mesg);
+						return false;
+					}
+
+					/* clean-up our target connection state for next table */
+					(void) copydb_close_snapshot(specs);
+
+					if (!copydb_set_snapshot(specs))
+					{
+						/* errors have already been logged */
+						free(mesg);
+						return false;
+					}
 				}
 				break;
 			}
@@ -459,17 +546,19 @@ copydb_table_data_worker(CopyDataSpec *specs)
 			default:
 			{
 				log_error("Received unknown message type %ld on table queue %d",
-						  mesg.type,
+						  mesg->type,
 						  specs->copyQueue.qId);
 				break;
 			}
 		}
+
+		free(mesg);
 	}
 
 	/* terminate our connection to the source database now */
 	(void) copydb_close_snapshot(specs);
 
-	return false;
+	return errors == 0;
 }
 
 
@@ -495,11 +584,10 @@ copydb_copy_data_by_oid(CopyDataSpec *specs, uint32_t oid, uint32_t part)
 
 	if (table->partsArray.count < part)
 	{
-		log_error("Failed to find part %d for table \"%s\".\"%s\" (%u), "
+		log_error("Failed to find part %d for table %s (%u), "
 				  "which has only %d parts",
 				  part,
-				  table->nspname,
-				  table->relname,
+				  table->qname,
 				  table->oid,
 				  table->partsArray.count);
 		return false;
@@ -513,12 +601,27 @@ copydb_copy_data_by_oid(CopyDataSpec *specs, uint32_t oid, uint32_t part)
 		return false;
 	}
 
-	log_trace("copydb_copy_data_by_oid: %u \"%s.%s\", part %d",
+	log_trace("copydb_copy_data_by_oid: %u %s, part %d",
 			  oid,
-			  table->nspname,
-			  table->relname,
+			  table->qname,
 			  part);
 
+	char psTitle[BUFSIZE] = { 0 };
+
+	if (table->partsArray.count > 0)
+	{
+		sformat(psTitle, sizeof(psTitle), "pgcopydb: copy %s [%d/%d]",
+				table->qname,
+				part + 1,
+				table->partsArray.count);
+	}
+	else
+	{
+		sformat(psTitle, sizeof(psTitle), "pgcopydb: copy %s", table->qname);
+	}
+
+	(void) set_ps_title(psTitle);
+
 	/*
 	 * Skip tables that have been entirely done already on a previous run.
 	 */
@@ -602,9 +705,8 @@ copydb_copy_data_by_oid(CopyDataSpec *specs, uint32_t oid, uint32_t part)
 
 					if (!vacuum_add_table(specs, sourceTable->oid))
 					{
-						log_error("Failed to queue VACUUM ANALYZE \"%s\".\"%s\" [%u]",
-								  sourceTable->nspname,
-								  sourceTable->relname,
+						log_error("Failed to queue VACUUM ANALYZE %s [%u]",
+								  sourceTable->qname,
 								  sourceTable->oid);
 						return false;
 					}
@@ -1074,6 +1176,7 @@ copydb_prepare_copy_query(CopyTableDataSpec *tableSpecs,
 						  bool source)
 {
 	SourceTable *table = tableSpecs->sourceTable;
+	bool isFirst = true;
 
 	if (source)
 	{
@@ -1085,17 +1188,29 @@ copydb_prepare_copy_query(CopyTableDataSpec *tableSpecs,
 
 		for (int i = 0; i < table->attributes.count; i++)
 		{
-			char *attname = table->attributes.array[i].attname;
+			SourceTableAttribute *attribute = &(table->attributes.array[i]);
+			char *attname = attribute->attname;
 
-			if (i > 0)
+			/* Generated columns cannot be used in COPY */
+			if (attribute->attisgenerated)
+			{
+				log_notice("Skipping %s in COPY as it is a generated column", attname);
+				continue;
+			}
+
+			if (!isFirst)
 			{
 				appendPQExpBufferStr(query, ", ");
 			}
+			else
+			{
+				isFirst = false;
+			}
 
-			appendPQExpBuffer(query, "\"%s\"", attname);
+			appendPQExpBuffer(query, "%s", attname);
 		}
 
-		appendPQExpBuffer(query, " FROM %s ", tableSpecs->sourceTable->qname);
+		appendPQExpBuffer(query, " FROM only %s ", tableSpecs->sourceTable->qname);
 
 		/*
 		 * On a source COPY query we might want to add filtering.
@@ -1107,11 +1222,32 @@ copydb_prepare_copy_query(CopyTableDataSpec *tableSpecs,
 			 * overlapping, so we can use the BETWEEN operator to select our source
 			 * rows in the COPY sub-query.
 			 */
-			appendPQExpBuffer(query,
-							  " WHERE \"%s\" BETWEEN %lld AND %lld ",
-							  tableSpecs->part.partKey,
-							  (long long) tableSpecs->part.min,
-							  (long long) tableSpecs->part.max);
+			if (streq(tableSpecs->part.partKey, "ctid"))
+			{
+				if (tableSpecs->part.max == -1)
+				{
+					/* the last part for ctid splits covers "extra" relpages */
+					appendPQExpBuffer(query,
+									  " WHERE ctid >= '(%lld,0)'::tid",
+									  (long long) tableSpecs->part.min + 1);
+				}
+				else
+				{
+					appendPQExpBuffer(query,
+									  " WHERE ctid >= '(%lld,0)'::tid"
+									  " and ctid < '(%lld,0)'::tid",
+									  (long long) tableSpecs->part.min,
+									  (long long) tableSpecs->part.max + 1);
+				}
+			}
+			else
+			{
+				appendPQExpBuffer(query,
+								  " WHERE %s BETWEEN %lld AND %lld ",
+								  tableSpecs->part.partKey,
+								  (long long) tableSpecs->part.min,
+								  (long long) tableSpecs->part.max);
+			}
 		}
 
 		appendPQExpBufferStr(query, ")");
@@ -1125,18 +1261,27 @@ copydb_prepare_copy_query(CopyTableDataSpec *tableSpecs,
 
 		for (int i = 0; i < table->attributes.count; i++)
 		{
-			char *attname = table->attributes.array[i].attname;
+			SourceTableAttribute *attribute = &(table->attributes.array[i]);
+			char *attname = attribute->attname;
 
-			if (i == 0)
+			/* Generated columns cannot be used in COPY */
+			if (attribute->attisgenerated)
+			{
+				log_notice("Skipping %s in COPY as it is a generated column", attname);
+				continue;
+			}
+
+			if (isFirst)
 			{
 				appendPQExpBufferStr(query, "(");
+				isFirst = false;
 			}
 			else
 			{
 				appendPQExpBufferStr(query, ", ");
 			}
 
-			appendPQExpBuffer(query, "\"%s\"", attname);
+			appendPQExpBuffer(query, "%s", attname);
 		}
 
 		if (table->attributes.count > 0)
