@@ -13,6 +13,8 @@ set -e
 # make sure source and target databases are ready
 pgcopydb ping
 
+sql="ALTER DATABASE postgres SET search_path TO public, abc;"
+psql -a -d "${PGCOPYDB_SOURCE_PGURI}" -c "${sql}"
 psql -a -d "${PGCOPYDB_SOURCE_PGURI}" -1 -f ./setup/setup.sql
 
 # create the target needed collation manually for the test
@@ -25,6 +27,7 @@ create collation if not exists mycol
 EOF
 
 # pgcopydb fork uses the environment variables
+export PGCOPYDB_SPLIT_TABLES_LARGER_THAN="2MB"
 pgcopydb fork --skip-collations --fail-fast --notice
 
 # now compare the output of running the SQL command with what's expected
@@ -55,5 +58,6 @@ do
     test -f $e || cat $r
     # exclude logs, whitespaces and blank lines
     DIFFOPTS='-B -w -I INFO -I WARN'
+    diff ${DIFFOPTS} $e $r || cat $r
     diff ${DIFFOPTS} $e $r || exit 1
 done
